SystemOrganization addCategory: #'Path-Contracts-Builder'!
SystemOrganization addCategory: #'Path-Contracts-Core'!
SystemOrganization addCategory: #'Path-Contracts-Evaluation'!
SystemOrganization addCategory: #'Path-Contracts-Exceptions'!
SystemOrganization addCategory: #'Path-Contracts-Kernel'!

!Exception methodsFor: '*Path-Contracts' stamp: 'mp 3/15/2012 09:50:32.036'!
isContractViolation
	^ false! !

!BasicClassOrganizer methodsFor: '*Path-Contracts' stamp: 'mp 5/15/2013 10:56:17.177'!
invariantGroups
	^ SContractInvariantRegistry invariantGroupAt: self
! !

!BasicClassOrganizer methodsFor: '*Path-Contracts' stamp: 'mp 5/15/2013 11:01:21.109'!
invariantGroups: anObject
	SContractInvariantRegistry
		invariantGroupAt: self
		put: anObject.

! !

!BasicClassOrganizer methodsFor: '*Path-Contracts' stamp: 'mp 5/15/2013 11:02:09.535'!
invariantMethod
	^ SContractInvariantRegistry invariantMethodAt: self! !

!BasicClassOrganizer methodsFor: '*Path-Contracts' stamp: 'mp 5/15/2013 11:02:43.088'!
invariantMethod: aMethod
	"Groups are cached for performance reasons (accessing pragmas
	seems to be very time expensive)"
	self invariantGroups: (SContract extractGroupsFrom: aMethod).
	SContractInvariantRegistry
		invariantMethodAt: self
		put: aMethod.! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'ls 1/6/2012 15:13:01.696'!
forkWithContracts
	
	^ self forkWithContracts: SProject allProjectNames! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'mp 5/14/2013 14:57:50.11'!
forkWithContracts: activeGroups
	| process policy |
	process := self newProcess.
	policy := SExecutionPolicy executionEnabled.
	policy activeGroups: activeGroups.
	process 
		contractExecutionPolicy: policy;
		name: 'Contract Execution: ', self outerContext asString;
		priority: Processor userBackgroundPriority.
	^ process resume! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'mp 5/14/2013 14:57:52.001'!
forkWithoutContracts
	| process |
	process := self newProcess.
	process contractExecutionPolicy: SExecutionPolicy executionDisabled.
	^ process resume! !

!BlockClosure methodsFor: '*Path-Contracts-override' stamp: 'ls 1/13/2012 13:01:34.932'!
newProcess
	"Answer a Process running the code in the receiver. The process is not 
	scheduled."
	| process |
	<primitive: 19> "Simulation guard"
	process := Process
		forContext: 
			[self value.
			Processor terminateActive] asContext
		priority: Processor activePriority.
	process contractExecutionPolicy: Processor activeProcess contractExecutionPolicy.
	^ process! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'ls 1/13/2012 12:08:22.096'!
runWithContracts
	
	^ self runWithContracts: SProject allProjectNames! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'mp 5/14/2013 14:57:53.458'!
runWithContracts: activeGroups
	| policy oldPolicy |
	oldPolicy := Processor activeProcess contractExecutionPolicy.
	policy := SExecutionPolicy executionEnabled activeGroups: activeGroups.
	Processor activeProcess contractExecutionPolicy: policy.
	^ self ensure: [Processor activeProcess contractExecutionPolicy: oldPolicy]! !

!BlockClosure methodsFor: '*Path-Contracts' stamp: 'mp 5/14/2013 14:57:55.028'!
runWithoutContracts
	| oldPolicy |
	oldPolicy := Processor activeProcess contractExecutionPolicy.
	Processor activeProcess contractExecutionPolicy: SExecutionPolicy executionDisabled.
	^ self ensure: [Processor activeProcess contractExecutionPolicy: oldPolicy]! !

!Process methodsFor: '*Path-Contracts' stamp: 'mp 5/14/2013 18:05:50.476'!
contractExecutionPolicy

	^ SContractExecutionPolicyRegistry
		atProcess: self
		ifAbsent: [SExecutionPolicy default]! !

!Process methodsFor: '*Path-Contracts' stamp: 'mp 5/15/2013 10:33:26.022'!
contractExecutionPolicy: aDCExecutionPolicy

	SContractExecutionPolicyRegistry
		atProcess: self
		put: aDCExecutionPolicy.! !

!Behavior methodsFor: '*Path-Contracts' stamp: 'mp 5/29/2013 17:26:28.891'!
argumentTemplate
	
	^ 'SContract expect: <ARGUMENT> toBeKindOf: <TYPE>.'! !

!Behavior methodsFor: '*Path-Contracts' stamp: 'mp 5/29/2013 17:26:22.98'!
contractPragmas
	<pragmas: #instance>
	^ #(#preconditions:  #postconditions: #invariants:)! !

!Behavior methodsFor: '*Path-Contracts' stamp: 'mp 5/29/2013 17:26:16.417'!
printContractFor: anArgumentName with: aBuilder
	| contract |
	anArgumentName ifNil: [^ self].
	(SContract blacklistedTypes includes: self) ifTrue: [^ self].
	contract := self argumentTemplate 
		copyReplaceTokens: '<ARGUMENT>' with: anArgumentName.
	contract := contract copyReplaceTokens: '<TYPE>' with: self asString.
	aBuilder printSourceLine: contract! !

MwMethodWrapper subclass: #SContractMutationWrapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Evaluation'!

!SContractMutationWrapper commentStamp: '<historical>' prior: 0!
A DCMutationWrapper is used to inject possible defects via the mutation engine.

Instance Variables


All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SContractMutationWrapper methodsFor: 'as yet unclassified' stamp: 'ls 1/5/2012 12:16:30.973'!
valueWithReceiver: anObject arguments: anArrayOfObjects

	^ anObject! !

MwMethodWrapper subclass: #SContractWrapper
	instanceVariableNames: 'beforeMethod afterMethod result oldObject requiresOldObject preconditionGroups postconditionGroups'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Core'!

!SContractWrapper commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SContractWrapper methodsFor: 'accessing' stamp: 'or 11/30/2011 19:45:35.702'!
afterMethod
	^ afterMethod! !

!SContractWrapper methodsFor: 'accessing' stamp: 'mp 5/14/2013 14:57:13.477'!
afterMethod: aMethod
	"Groups are cached for performance reasons (accessing pragmas
	seems to be very time expensive) Same goes for old object."
	self
		postconditionGroups: (SContract extractGroupsFrom: aMethod);
		requiresOldObject: (SContract requiresOldObjectFor: aMethod).
	afterMethod := aMethod.! !

!SContractWrapper methodsFor: 'accessing' stamp: 'or 11/23/2011 13:47:09.967'!
beforeMethod
	^ beforeMethod! !

!SContractWrapper methodsFor: 'accessing' stamp: 'mp 5/14/2013 14:57:15.122'!
beforeMethod: aMethod
	"Groups are cached for performance reasons (accessing pragmas
	seems to be very time expensive)"
	self preconditionGroups: (SContract extractGroupsFrom: aMethod).
	beforeMethod := aMethod! !

!SContractWrapper methodsFor: 'execution' stamp: 'ls 1/18/2012 17:40:04.198'!
containsActiveGroup: aGroup policy: policy
	aGroup isEmpty ifTrue: [^ policy activeGroups includes: #null].
	^  (aGroup anySatisfy: [:group | policy activeGroups includes: group])! !

!SContractWrapper methodsFor: 'compiled method' stamp: 'or 1/31/2012 14:11:46.619'!
getSourceFor: selector in: class
	"Polymorphic with CompiledMethod"
	^ self clientMethod getSourceFor: selector in: class! !

!SContractWrapper methodsFor: 'execution' stamp: 'or 1/18/2012 16:33:57.572'!
invariantGroups
	^ self wrappedClass organization invariantGroups! !

!SContractWrapper methodsFor: 'execution' stamp: 'or 1/31/2012 14:15:23.587'!
invariantsWithReceiver: anObject policy: policy
	"Dynamic Contract: Invocation of preconditions for executed method"
	(policy invariantsActive and: 
	[self containsActiveGroup: self invariantGroups policy: policy]) ifTrue: 
		[anObject class organization invariantMethod 
			valueWithReceiver: anObject arguments: {}] ! !

!SContractWrapper methodsFor: 'execution' stamp: 'ls 1/6/2012 13:58:06.379'!
isContractWrapper

	^ true! !

!SContractWrapper methodsFor: 'accessing' stamp: 'ls 1/14/2012 22:45:01.38'!
oldObject

	^ oldObject! !

!SContractWrapper methodsFor: 'accessing' stamp: 'ls 1/14/2012 22:44:56.78'!
oldObject: anObject

	oldObject := anObject! !

!SContractWrapper methodsFor: 'accessing' stamp: 'ls 1/18/2012 17:35:44.002'!
postconditionGroups

	^ postconditionGroups! !

!SContractWrapper methodsFor: 'accessing' stamp: 'ls 1/18/2012 17:35:37.369'!
postconditionGroups: anObject

	postconditionGroups := anObject! !

!SContractWrapper methodsFor: 'execution' stamp: 'or 1/31/2012 14:15:42.001'!
postconditionsWithReceiver: anObject arguments: args policy: policy
	"Dynamic Contract: Invocation of postconditions for executed method"
	(policy postconditionsActive and: 
	[self containsActiveGroup: self postconditionGroups policy: policy]) ifTrue:
		[self afterMethod valueWithReceiver: anObject arguments: args]! !

!SContractWrapper methodsFor: 'accessing' stamp: 'ls 1/18/2012 17:35:31.382'!
preconditionGroups

	^ preconditionGroups! !

!SContractWrapper methodsFor: 'accessing' stamp: 'ls 1/18/2012 17:35:24.524'!
preconditionGroups: anObject

	preconditionGroups := anObject! !

!SContractWrapper methodsFor: 'execution' stamp: 'or 1/31/2012 14:15:46.731'!
preconditionsWithReceiver: anObject arguments: args policy: policy
	"Dynamic Contract: Invocation of preconditions for executed method"
	(policy preconditionsActive and: 
	[self containsActiveGroup: self preconditionGroups policy: policy]) ifTrue: 
		[self beforeMethod valueWithReceiver: anObject arguments: args]! !

!SContractWrapper methodsFor: 'accessing' stamp: 'mp 7/16/2013 11:09:39.701'!
requiresOldObject
	"Lazy initialization should only take place if a compiled method sneaked into the 	system	which has not been compiled through tools with Dynamic Contract support"
	^ requiresOldObject ifNil: 
		[requiresOldObject := SContract requiresOldObjectFor: self afterMethod]! !

!SContractWrapper methodsFor: 'accessing' stamp: 'ls 1/17/2012 19:38:26.819'!
requiresOldObject: anObject

	requiresOldObject := anObject! !

!SContractWrapper methodsFor: 'accessing' stamp: 'ls 12/6/2011 23:04:46.119'!
result

	^ result! !

!SContractWrapper methodsFor: 'accessing' stamp: 'ls 12/6/2011 23:04:51.205'!
result: anObject

	result := anObject! !

!SContractWrapper methodsFor: 'execution' stamp: 'or 1/31/2012 14:17:49.292'!
valueWithReceiver: anObject arguments: anArray
	"Dynamic Contracts use this method to enforce contracts on the actual method"
	| policy |
	policy := Processor activeProcess contractExecutionPolicy copy.
	policy ifNotActive: 
		[^ self clientMethod valueWithReceiver: anObject arguments: anArray].
	self requiresOldObject ifTrue: [self oldObject: anObject veryDeepCopy].	
	
	[self 
		invariantsWithReceiver: anObject policy: policy;
		preconditionsWithReceiver: anObject arguments: anArray policy: policy
	] runWithoutContracts.
	
	self result: (self clientMethod valueWithReceiver: anObject arguments: anArray).
	
	[self 
		postconditionsWithReceiver: anObject arguments: anArray policy: policy; 
		invariantsWithReceiver: anObject policy: policy
	] runWithoutContracts.
	
	^ self result! !

!Object methodsFor: '*Path-Contracts' stamp: 'ls 1/6/2012 13:57:20.542'!
isContractWrapper
	
	^ false! !

Object subclass: #SContract
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Core'!

!SContract commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SContract class methodsFor: 'accessing' stamp: 'or 1/31/2012 13:51:40.35'!
blacklistedTypes
	"No contracts will be generated that require an object to be kind of a type
	listed below "
	^ { Object. SOMUniverse}! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:36:57.619'!
check: anObject and: aClass IfPossible: aBlock
	anObject ifNil: [^ self].
	(anObject class includesBehavior: aClass) ifFalse: [^ self].
	((anObject class includesBehavior: String) and: [aClass = Collection]) ifTrue: [^ self].
	aBlock value.! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 8/7/2013 11:27:28.777'!
checkSpellingOf: aString
	^ DTRHunspell instance spell: aString asString! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 16:58:20.921'!
compare: anObject with: anotherObject
	^ anotherObject isSymbol
			ifTrue: [anObject perform: anotherObject]
			ifFalse: [anObject = anotherObject]! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 1/18/2013 14:34:43.1'!
contractViolationFor: aClass and: anObject
	| context violation |
	context := self identifyWrapperContext.
	violation := aClass
		on: (self getClientMethod: context) 
		kindOf: (self getContractType: context).
	self setVariableNameIn: violation for: anObject with: context.
	^ violation! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 6/7/2013 10:14:21.73'!
convertToComparableValue: anObject type: aClass
	anObject isString
		ifTrue: [
			(aClass includesBehavior: DateAndTime)
				ifTrue: [^ DateAndTime fromString: anObject].
			(aClass includesBehavior: Duration)
				ifTrue: [^ Duration fromString: anObject]].
	^ anObject! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 5/14/2013 15:02:20.342'!
expect: anObject ofType: aClass is: anotherObject
	self check: anObject and: aClass IfPossible: [
		(self compare: anObject with: anotherObject)
			ifFalse: [
				(self contractViolationFor: SPropertyContractViolation and: anObject) 
					expected: true;
					givenProperty: anObject;
					expectedProperty: anotherObject;
					signal]].! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 5/14/2013 15:01:59.747'!
expect: anObject ofType: aClass isBetween: min and: max
	| minValue maxValue |
	self check: anObject and: aClass IfPossible: [
		minValue := self convertToComparableValue: min type: aClass.
		maxValue :=  self convertToComparableValue: max type: aClass.
		(anObject between: minValue and: maxValue)
			ifFalse: [
				(self contractViolationFor: SRangeContractViolation and: anObject) 
					expectedMin: min;
					expectedMax: max;
					givenObject: anObject;
					property: nil;
					signal]].! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 5/14/2013 15:02:22.472'!
expect: anObject ofType: aClass isNot: anotherObject
	self check: anObject and: aClass IfPossible: [
		(self compare: anObject with: anotherObject)
			ifTrue: [
				(self contractViolationFor: SPropertyContractViolation and: anObject) 
					expected: false;
					givenProperty: anObject;
					expectedProperty: anotherObject;
					signal]].! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 5/14/2013 15:02:56.015'!
expect: anObject ofType: aClass property: aSymbol includes: aCollection
	self check: anObject and: aClass IfPossible: [
		(aCollection includes: (self getPropertyFrom: anObject with: aSymbol))
			ifFalse: [
				(self contractViolationFor: SIncludesContractViolation and: anObject) 
					expectedCollection: aCollection;
					givenObject: anObject;
					property: aSymbol;
					signal]].! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 5/14/2013 15:02:02.287'!
expect: anObject ofType: aClass property: aSymbol isBetween: min and: max
	| minValue maxValue |
	self check: anObject and: aClass IfPossible: [
		minValue := self convertToComparableValue: min type: aClass.
		maxValue :=  self convertToComparableValue: max type: aClass.	
		((self getPropertyFrom: anObject with: aSymbol) 
			between: minValue and: maxValue)
				ifFalse: [
				(self contractViolationFor: SRangeContractViolation and: anObject) 
					expectedMin: min;
					expectedMax: max;
					givenObject: anObject;
					property: aSymbol;
					signal]].! !

!SContract class methodsFor: 'type contracts' stamp: 'mp 5/14/2013 15:01:03.595'!
expect: anObject toBeKindOf: aType
	anObject ifNil: [^ self].
	(anObject isKindOf: aType) 
		ifFalse: [(self contractViolationFor: STypeContractViolation and: anObject)
					messageText: 'Type violation';
					givenType: anObject class;
					expectedType: aType;
					signal].
	! !

!SContract class methodsFor: 'type contracts' stamp: 'mp 5/10/2012 16:04:04.612'!
expectAllElementsOf: aCollection toBeKindOf: aType
	aCollection isCollection ifFalse: [^ self].
	aCollection ifNil: [^ self].
	aCollection do: [:each | self expect: each toBeKindOf: aType]! !

!SContract class methodsFor: 'type contracts' stamp: 'mp 5/10/2012 16:03:55.252'!
expectAllKeysOf: aDictionary toBeKindOf: aType
	aDictionary isDictionary ifFalse: [^ self].
	aDictionary ifNil: [^ self].
	aDictionary keysDo: [:each | self expect: each toBeKindOf: aType]! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 5/14/2013 15:01:23.938'!
expectCorrectSpellingOf: aString
	self check: aString and: String IfPossible: [
		(self checkSpellingOf: aString)
			ifFalse: [
				(self contractViolationFor: SSpellingContractViolation and: aString) 
					givenString: aString;
					signal]].! !

!SContract class methodsFor: 'type contracts' stamp: 'mp 5/10/2012 15:58:05.966'!
expectElementsOf: aCollection toBeKindOf: aType
	aCollection isCollection ifFalse: [^ self].
	aCollection ifNil: [^ self].
	aCollection do: [:each | self expect: each toBeKindOf: aType. ^ self]! !

!SContract class methodsFor: 'type contracts' stamp: 'mp 5/10/2012 16:03:33.492'!
expectKeysOf: aDictionary toBeKindOf: aType
	aDictionary isDictionary ifFalse: [^ self].
	aDictionary ifNil: [^ self].
	aDictionary keysDo: [:each | self expect: each toBeKindOf: aType. ^ self]! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 5/14/2013 15:02:41.234'!
expectNotNil: anObject
	anObject ifNil: [(self contractViolationFor: SNilContractViolation and: anObject) signal]! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 5/14/2013 15:01:41.37'!
expectSize: anObject ofType: aClass isLessThan: aNumber
	self check: anObject and: aClass IfPossible: [
		(anObject size <= aNumber)
			ifFalse: [
				(self contractViolationFor: SSizeContractViolation and: anObject) 
					givenSize: anObject size;
					expectedSize: aNumber;
					signal]].! !

!SContract class methodsFor: 'Contract enforcement' stamp: 'or 1/31/2012 13:56:28.022'!
extractGroupsFrom: aContractMethod

	| pragma |
	aContractMethod ifNil: [^ nil].
	"This works since each contract method contains exactly one of our pragmas"
	pragma := (aContractMethod pragmas select: [:p | 
		Object contractPragmas includes: p keyword]) first.
	^ pragma arguments first ifNil: [Set new]! !

!SContract class methodsFor: 'exceptions-helper' stamp: 'mp 3/15/2012 11:20:33.648'!
getClientMethod: aContext
	^ aContext receiver clientMethod asMethodReference! !

!SContract class methodsFor: 'exceptions-helper' stamp: 'mp 3/15/2012 11:26:44.919'!
getContractType: aContext
	| wrapperSelector |
	wrapperSelector := aContext method selector.
	(wrapperSelector beginsWith: #preconditions)
		ifTrue: [^ #precondition].
	(wrapperSelector beginsWith: #invariants)
		ifTrue: [^ #invariant].
	(wrapperSelector beginsWith: #postconditions)
		ifTrue: [^ #postcondition].				! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 3/21/2012 17:22:41.818'!
getPropertyFrom: anObject with: aSymbol
	^ aSymbol substrings 
		inject: anObject 
		into: [:result :each | result perform: each asSymbol]! !

!SContract class methodsFor: 'exceptions-helper' stamp: 'mp 1/18/2013 14:21:02.798'!
identifyWrapperContext 
	| context |
	context := thisContext.
	[context notNil and: [context receiver isContractWrapper not]] 
		whileTrue: [context := context sender].
	context ifNil: [self halt "should not be triggered but if we can debug it!!"].
	^ context! !

!SContract class methodsFor: 'Contract enforcement' stamp: 'or 1/18/2012 16:51:40.346'!
oldObject
	"You should only call this method from within a postcondition"
	^ thisContext sender sender sender receiver oldObject! !

!SContract class methodsFor: 'Contract enforcement' stamp: 'ls 1/17/2012 19:37:23.562'!
requiresOldObjectFor: aPostconditionMethod

	^ aPostconditionMethod sendsSelector: #oldObject! !

!SContract class methodsFor: 'Contract enforcement' stamp: 'ls 1/20/2012 14:13:16.984'!
result
	"You should only call this method from within a postcondition"
	^ thisContext sender sender sender receiver result! !

!SContract class methodsFor: 'exceptions-helper' stamp: 'mp 5/30/2013 12:06:16.949'!
setVariableNameIn: aViolation for: anObject with: aContext
	| arguments receiver allInstVarValues |
	aViolation kindOf == #precondition
		ifTrue: [	arguments := aContext at: 2.
				(arguments select: [:each | each == anObject]) size = 1
					ifTrue:[aViolation variableName: 
							(aContext receiver clientMethod methodNode arguments 
								at: (arguments indexOf: anObject)) key]].
	aViolation kindOf == #invariant
		ifTrue: [receiver := aContext at: 1.
			allInstVarValues := receiver class allInstVarNames 
									collect: [:each | receiver instVarNamed: each].
			(allInstVarValues select: [:each | each == anObject]) size = 1
				ifTrue: [	aViolation variableName: 
							(receiver class allInstVarNames 
								at: (allInstVarValues indexOf: anObject))]].! !

!SContract class methodsFor: 'value range contracts' stamp: 'mp 5/14/2013 15:00:28.591'!
signalContractViolation
	| context |
	context := self identifyWrapperContext.
	SContractViolation
		signalOn: (self getClientMethod: context) 
		kindOf: (self getContractType: context).! !

Object subclass: #SContractBuilder
	instanceVariableNames: 'selector arguments groups typeInfo valueRangeInfo sources compiledMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Builder'!

!SContractBuilder commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SContractBuilder methodsFor: 'configuration' stamp: 'ls 12/27/2011 16:38:29.271'!
addGroup: aSymbol

	self groups addIfNotPresent: aSymbol asSymbol! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 13:19:40.507'!
arguments

	^ arguments ifNil: [arguments := OrderedCollection new]! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:12:59.192'!
arguments: anObject

	arguments := anObject! !

!SContractBuilder methodsFor: 'building' stamp: 'ls 12/27/2011 00:51:13.315'!
build
	
	self
		buildSelector;
		buildPragma;
		buildContracts! !

!SContractBuilder methodsFor: 'building' stamp: 'ls 12/27/2011 01:02:18.82'!
buildAndStoreIn: aClass
	
	self	 
		build;
		storeIn: aClass! !

!SContractBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 11:48:18.453'!
buildContracts
	self typeInfo 
		ifNotNil: [self 
					printSourceLine: '"Type Contracts"';
					buildTypeContracts].
	self valueRangeInfo
		ifNotNil: [self 
					printSourceLine: '"Value Range Contracts"';
					buildValueRangeContracts].! !

!SContractBuilder methodsFor: 'building' stamp: 'or 1/31/2012 13:59:51.993'!
buildPragma

	| pragma groups |
	groups := WriteStream on: String new.
	self groups do: [:group | group storeOn: groups] separatedBy: [groups space].
	pragma := self pragmaTemplate 
		copyReplaceTokens: '<GROUPS>' with: groups contents.
	self printSourceLine: pragma! !

!SContractBuilder methodsFor: 'building' stamp: 'ls 12/27/2011 12:57:10.622'!
buildSelector

	| selectorWithArgs |
	selectorWithArgs := WriteStream on: String new.
	self selector numArgs = 0 ifTrue: [
		self sources nextPutAll: self selector keywords first.
		^ self].
	
	(1 to: self selector numArgs) do: [:nr |
		selectorWithArgs
			nextPutAll: (self selector keywords at: nr); space;
			nextPutAll: (arguments at: nr); space].
	self sources nextPutAll: selectorWithArgs contents! !

!SContractBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 11:41:59.771'!
buildTypeContracts
	self subclassResponsibility.! !

!SContractBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 11:42:12.451'!
buildValueRangeContracts
	self subclassResponsibility.! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:05.137'!
compiledMethod

	^ compiledMethod! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 01:04:52.437'!
compiledMethod: anObject

	compiledMethod := anObject! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 13:19:28.101'!
groups

	^ groups ifNil: [groups := OrderedCollection new]! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:14.375'!
groups: anObject

	groups := anObject! !

!SContractBuilder methodsFor: 'templates' stamp: 'ls 12/27/2011 00:20:44.572'!
pragmaTemplate

	^ self subclassResponsibility! !

!SContractBuilder methodsFor: 'templates' stamp: 'mp 3/20/2012 16:51:39.945'!
printContractsFor: anArgumentName valueRanges: aCollectionOfRanges

	aCollectionOfRanges do: [:each | each printContractFor: anArgumentName with: self].		
! !

!SContractBuilder methodsFor: 'templates' stamp: 'ls 1/5/2012 12:17:01.809'!
printSourceLine: aString

	self sources
		nextPut: Character cr;
		nextPut: Character tab;
		nextPutAll: aString! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/26/2011 23:39:19.669'!
selector

	^ selector! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:19.861'!
selector: anObject

	selector := anObject! !

!SContractBuilder methodsFor: 'configuration' stamp: 'ls 12/28/2011 17:14:45.308'!
setSelector: aSymbol argumentNames: aCollection

	self 
		selector: aSymbol;
		arguments: aCollection	! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 00:34:24.34'!
sources

	^ sources ifNil: [sources := WriteStream on: String new]! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:24.488'!
sources: anObject

	sources := anObject! !

!SContractBuilder methodsFor: 'building' stamp: 'ls 12/27/2011 12:55:23.391'!
storeIn: aClass

	| methodNode |
	methodNode := (aClass 
		compile:  self sources contents 
		classified: nil 
		notifying: nil 
		trailer: aClass defaultMethodTrailer 
		ifFail: [^ nil]).
	methodNode method 
		putSource: self sources contents 
		fromParseNode: methodNode node inFile: 2
		withPreamble: [:f | f cr; nextPut: $!!; nextChunkPut: 'Contract Method'; cr].
	self compiledMethod: methodNode method! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:34.042'!
typeInfo

	^ typeInfo! !

!SContractBuilder methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:13:29.901'!
typeInfo: anObject

	typeInfo := anObject! !

!SContractBuilder methodsFor: 'accessing' stamp: 'mp 3/20/2012 11:09:04.035'!
valueRangeInfo
	"Answer the value of valueRangeInfo"

	^ valueRangeInfo! !

!SContractBuilder methodsFor: 'accessing' stamp: 'mp 3/20/2012 11:09:04.047'!
valueRangeInfo: anObject
	"Set the value of valueRangeInfo"

	valueRangeInfo := anObject! !

SContractBuilder subclass: #SInvariantBuilder
	instanceVariableNames: 'instVars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Builder'!

!SInvariantBuilder commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SInvariantBuilder methodsFor: 'configuration' stamp: 'ls 12/27/2011 00:19:08.594'!
addInstVar: aName

	self instVars addIfNotPresent: aName! !

!SInvariantBuilder methodsFor: 'configuration' stamp: 'ls 12/27/2011 00:19:19.544'!
addInstVars: aCollection

	aCollection do: [:instVar | self addInstVar: instVar]! !

!SInvariantBuilder methodsFor: 'building' stamp: 'mp 1/21/2013 15:57:17.912'!
buildTypeContracts

	| memberType |
	self instVars withIndexDo: [:instVarName :i |
		memberType := self typeInfo at: i.
		memberType ifNotNil: [memberType printContractFor: instVarName with: self]].! !

!SInvariantBuilder methodsFor: 'building' stamp: 'mp 7/20/2012 11:02:27.193'!
buildValueRangeContracts
	| memberRanges ranges |
	self instVars withIndexDo: [:instVarName :i |
		memberRanges := self valueRangeInfo at: i.
		memberRanges ifNotNil: [
			"remove nil assertions from invariants"
			ranges :=  memberRanges asOrderedCollection 
				reject: [:each | each class == SDefaultRangeProperties].
			self printContractsFor: instVarName 
				valueRanges: ranges]]! !

!SInvariantBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 13:18:47.697'!
instVars
	"Answer the value of instVars"

	^ instVars ifNil: [instVars := OrderedCollection new]! !

!SInvariantBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 00:19:31.928'!
instVars: anObject
	"Set the value of instVars"

	instVars := anObject! !

!SInvariantBuilder methodsFor: 'accessing' stamp: 'ls 12/27/2011 00:20:28.141'!
pragmaTemplate

	^ '<invariants: #(<GROUPS>)>'! !

SContractBuilder subclass: #SPostconditionBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Builder'!

!SPostconditionBuilder commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SPostconditionBuilder methodsFor: 'building' stamp: 'mp 5/14/2013 15:04:40.065'!
buildTypeContracts
	self typeInfo ifNil: [^ self].
	self typeInfo printContractFor: '(SContract result)' with: self.! !

!SPostconditionBuilder methodsFor: 'building' stamp: 'mp 5/14/2013 15:04:41.859'!
buildValueRangeContracts
	self printContractsFor: 'SContract result' 
		valueRanges: self valueRangeInfo asOrderedCollection! !

!SPostconditionBuilder methodsFor: 'templates' stamp: 'ls 12/27/2011 00:24:26.155'!
pragmaTemplate

	^ '<postconditions: #(<GROUPS>)>'! !

SContractBuilder subclass: #SPreconditionBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Builder'!

!SPreconditionBuilder commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SPreconditionBuilder methodsFor: 'building' stamp: 'mp 1/21/2013 14:38:13.115'!
buildTypeContracts
	| argumentType |
	self arguments withIndexDo: [:arg :i | 
		argumentType := self typeInfo at: i.
		argumentType ifNotNil: [argumentType printContractFor: arg with: self]]! !

!SPreconditionBuilder methodsFor: 'building' stamp: 'mp 3/20/2012 13:44:43.143'!
buildValueRangeContracts
	| argumentRange |
	self arguments withIndexDo: [:arg :i | 
		argumentRange := self valueRangeInfo at: i.
		argumentRange ifNotNil: [
			self printContractsFor: arg 
				valueRanges: argumentRange asOrderedCollection]].! !

!SPreconditionBuilder methodsFor: 'templates' stamp: 'ls 12/27/2011 00:23:41.123'!
pragmaTemplate

	^ '<preconditions: #(<GROUPS>)>'! !

Object subclass: #SContractEvaluator
	instanceVariableNames: 'coveredProject pathMap coveredMethods testResults'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Evaluation'!

!SContractEvaluator commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SContractEvaluator class methodsFor: 'instance creation' stamp: 'or 1/16/2012 14:27:17.417'!
on: anSProject
	| instance |
	instance := self new.
	instance 
		coveredProject: anSProject.
	^instance! !

!SContractEvaluator class methodsFor: 'instance creation' stamp: 'or 1/16/2012 14:13:31.518'!
onCurrentSProject
	
	^ self on: SProject current! !

!SContractEvaluator class methodsFor: 'accessing' stamp: 'or 1/31/2012 14:33:57.274'!
resultHeader
	^ (Dictionary newFromPairs: {
		#tag. 'Contracts enabled'. 
		#methodReference. 'Mutated method'. 
		#testFails. 'Test failures'. 
		#testErrors. 'Test errors'. 
		#contractFails. 'Failing contracts'})! !

!SContractEvaluator methodsFor: 'csv file' stamp: 'ls 1/5/2012 13:51:56.377'!
convertCollectionToCSV: aCollection  on: aStream

	aCollection withIndexDo: [:row :index |
		aStream nextPutAll: index asString.
		aStream nextPut: $,.
		row do: [:column | 
			aStream nextPutAll: column asString.
			aStream nextPut: $,.].
		aStream cr; flush.].! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'ls 1/15/2012 14:49:46.697'!
coveredMethods
	"Answer the value of coveredMethods"

	^ coveredMethods! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'ls 1/15/2012 14:49:46.711'!
coveredMethods: anObject
	"Set the value of coveredMethods"

	coveredMethods := anObject! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'ls 1/5/2012 13:28:29.534'!
coveredProject

	^ coveredProject! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'ls 1/5/2012 13:28:34.768'!
coveredProject: anObject

	coveredProject := anObject! !

!SContractEvaluator methodsFor: 'public api' stamp: 'ls 1/5/2012 14:46:49.616'!
do: aBlock withCurrentProject: aProjectName
	| previousCurrentProject result |
	[previousCurrentProject := SProject current.
	SProject setCurrent: aProjectName.
	result := aBlock value]
		ensure: [SProject setCurrentProject: previousCurrentProject].
	^ result! !

!SContractEvaluator methodsFor: 'public api' stamp: 'ls 1/15/2012 14:55:46.882'!
evaluate

	^ self evaluateFromEntryPoint: (self methodsCoveredByTests first)! !

!SContractEvaluator methodsFor: 'public api' stamp: 'mp 5/14/2013 15:00:32.309'!
evaluateFromEntryPoint: aMethodReference
	"Start iterating at aMethodReference and mutate all covered methods"
	| mutationEngine contractFailures |
	self testResults: (OrderedCollection with: self class resultHeader).
	self do: 
		[mutationEngine := SContractMutationEngine on: self methodsCoveredByTests.
		mutationEngine mutationsDo: [:methodReference |
			"Transcript cr; show: 'Mutated ', methodReference asString; cr."
			[self pathMap runTests: #suiteAll] runWithoutContracts.
			self 
				evaluateTestStateOf: self pathMap testRunner 
				with: #nocontracts 
				forMethodReference: methodReference.
			contractFailures := 0.
			[[self pathMap runTests: #suiteAll] runWithContracts: 
				self coveredProject projectName] 
			on: SContractViolation  do: [:exception | 
				"Transcript show: 'Contract: ', exception messageText; cr."
				contractFailures := contractFailures + 1. 
				exception return: false].
			self 
				evaluateTestStateOf: self pathMap testRunner 
				withTag: #contracts 
				forMethodReference: methodReference 
				andContractFails: contractFailures
				
		] entryPoint: aMethodReference.
		] withCurrentProject: self coveredProject projectName.
	self storeCollection: self testResults asCSVFile: 'Evaluation ', TimeStamp now, '.csv'.
	self showTestResults.! !

!SContractEvaluator methodsFor: 'public api' stamp: 'or 1/31/2012 14:31:44.1'!
evaluateTestStateOf: aTestRunner with: aTag forMethodReference: methodReference
	^ self 
		evaluateTestStateOf: aTestRunner 
		withTag: aTag 
		forMethodReference: methodReference 
		andContractFails: 0! !

!SContractEvaluator methodsFor: 'public api' stamp: 'or 1/31/2012 14:32:39.033'!
evaluateTestStateOf: aTestRunner withTag: aTag forMethodReference: methodReference andContractFails: anInteger

	| failures errors results |
	failures := aTestRunner failedList size.
	errors := aTestRunner errorList size.
	results := Dictionary newFromPairs: 
		{#tag. aTag. 
		#methodReference. (methodReference actualClass name, '>>', methodReference methodSymbol) asSymbol. 
		#testFails. failures. 
		#testErrors. errors. 
		#contractFails. anInteger}.
	self testResults add: results.! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'mp 5/10/2013 16:52:46.523'!
initializePathMap

	| testRunner |
	testRunner := ToolBuilder build: SPathMapTestRunner new.
	self pathMap: testRunner model pathMap.
	self pathMap globalOptions
		switchShowAllMethods;
		switchShowMethodCategories;
		switchVisualizeIntermediateSteps.! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'mp 5/8/2012 11:27:32.8'!
methodsCoveredByTests

	^ self coveredMethods ifNil: 
		[self initializePathMap.
		self pathMap runTests: #suiteAll with: #testQuality.
		self coveredMethods: (self pathMap tracer getAllCoveredMethods keys 
		reject: [:each | ((each methodSymbol beginsWith: 'test') 
						or: [each compiledMethod isQuick])
						or: [each actualClass inheritsFrom: TestCase]]).
		^ self coveredMethods]! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'ls 1/6/2012 15:23:45.383'!
pathMap

	^ pathMap! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'ls 1/6/2012 15:23:50.17'!
pathMap: anObject

	pathMap := anObject! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'mp 7/16/2013 11:10:52.406'!
showTestResults
	| contracts noContracts contractFailed noDifference contractOnly noFail |
	
	contractFailed := 0.
	noDifference := 0.
	contractOnly := 0.
	noFail := 0.
	
	Transcript show: '--- Results (see also logFile) ---'; cr.
	noContracts := self testResults select: [:item | (item at: #tag) = #nocontracts].
	contracts := self testResults select: [:item | (item at: #tag) = #contracts].
	
	noContracts do: [:item | | withContract | 
		withContract := contracts detect: [:c | 
			((c at: #methodReference) = (item at: #methodReference))].
		"A Contract caught an anomaly"		
		((withContract at: #contractFails) > 0) 
		ifTrue:
			[(((item at: #testFails) = 0) and: [(item at: #testErrors) = 0]) 
			"The tests did not notice the anomaly"
			ifTrue: [contractOnly := contractOnly + 1]
			"The anomaly was caught before a test failed"
			ifFalse: [((withContract at: #testFails) = 0) 
				ifTrue: [contractFailed := contractFailed + 1]]]
		ifFalse: [(((withContract at: #testFails) = 0) and: 
			[(withContract at: #testErrors) = 0]) 
			"no one discovered the anomaly"
			ifTrue: [noFail := noFail + 1]
			ifFalse: [noDifference := noDifference + 1]]].
				
	Transcript show: 'Mutated ', ((self testResults size - 1) / 2) asString, ' methods.'; cr.
	Transcript show: 'No difference: ', noDifference asString, 
		' Contract failed: ', contractFailed, 
		' Only contract failed: ', contractOnly, 
		' Anomaly unnoticed: ', noFail; cr! !

!SContractEvaluator methodsFor: 'csv file' stamp: 'ls 1/5/2012 13:51:42.98'!
storeCollection: aCollection asCSVFile: aFileName
	| file |
	[file := FileDirectory default forceNewFileNamed: aFileName.
	 self convertCollectionToCSV: aCollection  on: file]
	ensure: [file close].! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'or 1/16/2012 13:52:18.13'!
testResults
	"Answer the value of testResults"

	^ testResults! !

!SContractEvaluator methodsFor: 'accessing' stamp: 'or 1/16/2012 13:52:18.191'!
testResults: anObject
	"Set the value of testResults"

	testResults := anObject! !

Object subclass: #SContractExecutionPolicyRegistry
	instanceVariableNames: ''
	classVariableNames: 'Processes'
	poolDictionaries: ''
	category: 'Path-Contracts-Kernel'!

!SContractExecutionPolicyRegistry commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SContractExecutionPolicyRegistry class methodsFor: 'accessing' stamp: 'mp 5/14/2013 18:03:35.839'!
atProcess: aProcess ifAbsent: aBlock
	^ Processes 
		at: aProcess
		ifAbsentPut: aBlock! !

!SContractExecutionPolicyRegistry class methodsFor: 'accessing' stamp: 'mp 5/14/2013 18:04:44.867'!
atProcess: aProcess put: aExecutionPolicy
	^ Processes 
		at: aProcess
		put: aExecutionPolicy! !

!SContractExecutionPolicyRegistry class methodsFor: 'class initialization' stamp: 'mp 5/14/2013 17:57:05.501'!
initialize
	"self initialize"
	super initialize.
	Processes := WeakKeyDictionary new.! !

Object subclass: #SContractInjector
	instanceVariableNames: 'methodsInScope typesBucket valueRangeBucket groupName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Builder'!

!SContractInjector commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SContractInjector class methodsFor: 'instance creation' stamp: 'mp 3/20/2012 10:59:07.453'!
covering: aSProject
	"returns a DCContractInjector instance for the given classes
	which will generate pre- and postconditions as well as invariants"
	| instance knownMethods |
	instance := self new.
	knownMethods := Dictionary new.
	aSProject allClasses do: [:class | knownMethods at: class put: class selectors].
	instance 
		methodsInScope: knownMethods;
		typesBucket: aSProject typesBucket;
		valueRangeBucket: aSProject valueRangeBucket;
		groupName: aSProject projectName.
		
	^ instance
! !

!SContractInjector class methodsFor: 'instance creation' stamp: 'ls 12/22/2011 16:29:52.15'!
coveringCurrentSProject
	
	^ self covering: SProject current! !

!SContractInjector methodsFor: 'public api' stamp: 'ls 12/23/2011 18:31:40.361'!
apply
	
	'Injecting Contracts'
		displayProgressAt: Display center
		from: 0 to: self classesInScope size
		during: [:bar | 1 to: self classesInScope size do: [:i |
				bar value: (self classesInScope at: i) printString.
				self injectContractsIn: (self classesInScope at: i).
				bar value: i]]! !

!SContractInjector methodsFor: 'accessing' stamp: 'ls 11/27/2011 12:36:21.881'!
classesInScope

	^ self methodsInScope keys! !

!SContractInjector methodsFor: 'contract creation' stamp: 'mp 5/14/2013 14:56:21.799'!
generateInvariantsIn: aClass

	| builder selector |
	builder := SInvariantBuilder new.
	self typesBucket 
		ifNotNil: [builder typeInfo: (self typesBucket membersFor: aClass)].
	self valueRangeBucket
		ifNotNil: [builder valueRangeInfo: (self valueRangeBucket membersFor: aClass)].
	selector := aClass asString copyReplaceAll: ' ' with: '' asTokens: false.
	builder
		setSelector: selector argumentNames: {};
		addGroup: self groupName;
		addInstVars: aClass allInstVarNames;
		buildAndStoreIn: aClass.
	^ builder compiledMethod
! !

!SContractInjector methodsFor: 'contract creation' stamp: 'mp 5/14/2013 14:56:05.596'!
generatePostconditionsOn: aSymbol in: aClass
	
	| argumentNames builder |
	argumentNames := (aClass methodDict at: aSymbol) methodNode arguments collect: 		[:a | a name].
	builder := SPostconditionBuilder new. 
	self typesBucket 
		ifNotNil: [builder typeInfo: (self typesBucket returnFor: 
										(aClass methodDict at: aSymbol))].
	self valueRangeBucket
		ifNotNil: [builder valueRangeInfo: (self valueRangeBucket returnFor: 
											(aClass methodDict at: aSymbol))].
	builder
		setSelector: aSymbol argumentNames: argumentNames;
		addGroup: self groupName;
		buildAndStoreIn: aClass.
	^ builder compiledMethod
	! !

!SContractInjector methodsFor: 'contract creation' stamp: 'mp 5/14/2013 14:55:49.987'!
generatePreconditionsOn: aSymbol in: aClass
	
	| argumentNames builder |
	argumentNames := (aClass methodDict at: aSymbol) methodNode arguments collect: 		[:a | a name].
	builder := SPreconditionBuilder new. 
	self typesBucket 
		ifNotNil: [builder typeInfo: (self typesBucket argumentsFor: 
												(aClass methodDict at: aSymbol))].
	self valueRangeBucket
		ifNotNil: [builder valueRangeInfo: (self valueRangeBucket argumentsFor: 												(aClass methodDict at: aSymbol))].
	builder 
		setSelector: aSymbol argumentNames: argumentNames;
		addGroup: self groupName;
		buildAndStoreIn: aClass.
	^ builder compiledMethod! !

!SContractInjector methodsFor: 'accessing' stamp: 'ls 12/27/2011 18:41:34.524'!
groupName

	^ groupName! !

!SContractInjector methodsFor: 'accessing' stamp: 'ls 12/27/2011 18:41:39.012'!
groupName: anObject

	groupName := anObject! !

!SContractInjector methodsFor: 'contract creation' stamp: 'ls 1/18/2012 12:50:36.066'!
injectContractsIn: aClass
	
	| selectors |
	aClass organization invariantMethod: (self generateInvariantsIn: aClass).
	selectors := self methodsInScope at: aClass.
	selectors do: [:selector | self injectContractsOn: selector in: aClass].
	"Class side"
	aClass class organization invariantMethod: (self generateInvariantsIn: aClass class).
	aClass class selectors do: [:selector | self injectContractsOn: selector in: aClass class]! !

!SContractInjector methodsFor: 'contract creation' stamp: 'mp 5/14/2013 14:58:17.863'!
injectContractsOn: aSelector in: aClass
	
	| methodWrapper afterMethod |
	"First, make sure that no nested contracts are generated"
	self removeContractsOn: aSelector in: aClass.
	methodWrapper := SContractWrapper on: aSelector inClass: aClass.
	afterMethod := (self generatePostconditionsOn: aSelector in: aClass).
	methodWrapper 
		beforeMethod: (self generatePreconditionsOn: aSelector in: aClass);
		afterMethod: afterMethod.
	methodWrapper install! !

!SContractInjector methodsFor: 'accessing' stamp: 'ls 12/23/2011 18:26:08.69'!
methodsInScope

	^ methodsInScope! !

!SContractInjector methodsFor: 'accessing' stamp: 'ls 12/23/2011 18:26:16.258'!
methodsInScope: anObject

	methodsInScope := anObject! !

!SContractInjector methodsFor: 'contract removal' stamp: 'or 1/31/2012 14:08:11.324'!
removeContractsIn: aClass

	| selectors |
	aClass organization invariantMethod: nil.
	selectors := self methodsInScope at: aClass.
	selectors do: [:selector | self removeContractsOn: selector in: aClass].
	
	aClass class organization invariantMethod: nil.
	aClass class selectors do: [:selector | 
		self removeContractsOn: selector in: aClass class]! !

!SContractInjector methodsFor: 'contract removal' stamp: 'ls 1/6/2012 13:59:13.859'!
removeContractsOn: aSelector in: aClass

	| method |
	method := aClass methodDictionary at: aSelector.
	method isContractWrapper ifTrue: [method uninstall]! !

!SContractInjector methodsFor: 'public api' stamp: 'ls 12/25/2011 17:41:13.815'!
revert
	'Removing Contracts'
		displayProgressAt: Display center
		from: 0 to: self classesInScope size
		during: [:bar | 1 to: self classesInScope size do: [:i |
				bar value: (self classesInScope at: i) printString.
				self removeContractsIn: (self classesInScope at: i).
				bar value: i]]! !

!SContractInjector methodsFor: 'accessing' stamp: 'ls 12/23/2011 18:12:31.907'!
typesBucket

	^ typesBucket! !

!SContractInjector methodsFor: 'accessing' stamp: 'ls 12/23/2011 18:12:25.612'!
typesBucket: anObject

	typesBucket := anObject! !

!SContractInjector methodsFor: 'accessing' stamp: 'mp 3/20/2012 10:58:40.477'!
valueRangeBucket
	"Answer the value of valueRangeBucket"

	^ valueRangeBucket! !

!SContractInjector methodsFor: 'accessing' stamp: 'mp 3/20/2012 10:58:40.488'!
valueRangeBucket: anObject
	"Set the value of valueRangeBucket"

	valueRangeBucket := anObject! !

Object subclass: #SContractInvariantRegistry
	instanceVariableNames: ''
	classVariableNames: 'InvariantGroups InvariantMethods'
	poolDictionaries: ''
	category: 'Path-Contracts-Kernel'!

!SContractInvariantRegistry commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SContractInvariantRegistry class methodsFor: 'class initialization' stamp: 'mp 5/15/2013 10:50:49.856'!
initialize
	"self initialize"
	super initialize.
	InvariantGroups := WeakKeyDictionary new.
	InvariantMethods := WeakKeyDictionary new.! !

!SContractInvariantRegistry class methodsFor: 'accessing' stamp: 'mp 5/15/2013 10:58:03.401'!
invariantGroupAt: aClassOrganizer 
	^ InvariantGroups 
		at: aClassOrganizer
		ifAbsent: nil! !

!SContractInvariantRegistry class methodsFor: 'accessing' stamp: 'mp 5/15/2013 11:01:12.946'!
invariantGroupAt: aClassOrganizer put: anObject
	InvariantGroups
		at: aClassOrganizer
		put: anObject! !

!SContractInvariantRegistry class methodsFor: 'accessing' stamp: 'mp 5/15/2013 11:00:33.794'!
invariantMethodAt: aClassOrganizer 
	^ InvariantMethods 
		at: aClassOrganizer
		ifAbsent: nil! !

!SContractInvariantRegistry class methodsFor: 'accessing' stamp: 'mp 5/15/2013 11:03:04.337'!
invariantMethodAt: aClassOrganizer put: aMethod
	InvariantMethods
		at: aClassOrganizer
		put: aMethod! !

Object subclass: #SContractMutationEngine
	instanceVariableNames: 'coveredMethods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Evaluation'!

!SContractMutationEngine commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SContractMutationEngine class methodsFor: 'instance creation' stamp: 'ls 1/5/2012 13:09:39.365'!
on: aMethodReferenceCollection

	^ self new coveredMethods: aMethodReferenceCollection! !

!SContractMutationEngine class methodsFor: 'public api' stamp: 'mp 5/14/2013 14:59:06.822'!
undoMutationsGlobally
	
	"First, do a garbage collection so no iteration takes place over no
	longer referenced SContractMutationWrapper instances"
	Smalltalk garbageCollect.
	SContractMutationWrapper allInstancesDo: [:instance |
		instance uninstall].! !

!SContractMutationEngine methodsFor: 'checking' stamp: 'or 1/31/2012 14:43:24.773'!
blacklistedMethods
	"Methods listed below may lead to an image freeze if mutated"
	^ { 
		MethodReference class: SOMMethodParser2 selector: #countBracket.
		MethodReference class: SOMClassParser2 selector: #classInstanceSwitch.
		MethodReference class: SOMClassParser2 selector: #foreignParseMethod.
		MethodReference class: SOMMethodParser2 selector: #unaryMsg.
		MethodReference class: SOMMethodParser2 selector: #binaryMsg.
		MethodReference class: SOMMethodParser2 selector: #assignment.
		MethodReference class: SOMMethodParser2 selector: #binaryChar.
		MethodReference class: SOMMethodParser2 selector: #method.
		MethodReference class: SOMArray selector: #at:.
		MethodReference class: SOMArray selector: #add:.
		MethodReference class: SOMFrame selector: #at:put:.
		MethodReference class: SOMInterpreter selector: #popFrame.
		MethodReference class: SOMInterpreter selector: #somFrame:.
		MethodReference class: SOMClassGenerationContext selector: #addFields:.
		MethodReference class: SOMFrame selector: #pop.
		MethodReference class: SOMFrame selector: #stackElementAtIndex:.
		MethodReference class: SOMInteger class selector: #maxInt.
		MethodReference class: SOMMethodGenerationContext selector: #addLocalIfAbsent:.
		MethodReference class: SOMMethodGenerationContext selector: #outer:.
		MethodReference class: SOMParser2 selector: #comment.
		MethodReference class: SOMVariableLookup selector: #isArgument:.
		MethodReference class: SOMVariableLookup selector: #lookupArgument.
		MethodReference class: SOMVariableLookup selector: #lookupField.
		MethodReference class: SOMVariableLookup selector: #lookupLocal.
		MethodReference class: SOMVariableLookup selector: #isLocal:.
		MethodReference class: SOMVariableLookup selector: #isField:.
		MethodReference class: SOMBlock selector: #cacheEvaluationMethodWithParameters:.
		MethodReference class: SOMBlock selector: #somBlockMethod:.
	}! !

!SContractMutationEngine methodsFor: 'accessing' stamp: 'ls 1/5/2012 12:19:44.904'!
coveredMethods

	^ coveredMethods! !

!SContractMutationEngine methodsFor: 'accessing' stamp: 'ls 1/5/2012 12:19:48.577'!
coveredMethods: anObject

	coveredMethods := anObject! !

!SContractMutationEngine methodsFor: 'checking' stamp: 'ls 1/15/2012 18:00:49.502'!
isReturningSelf: aMethodReference
	| method |
	method := aMethodReference compiledMethod.
	"^ method isReturnSelf or: [method scanFor: 16r78]"
	^ method isReturnSelf! !

!SContractMutationEngine methodsFor: 'public api' stamp: 'ls 1/15/2012 14:07:59.874'!
mutationsDo: aBlock
	
	^ self mutationsDo: aBlock entryPoint: self coveredMethods first! !

!SContractMutationEngine methodsFor: 'public api' stamp: 'mp 5/14/2013 14:59:17.91'!
mutationsDo: aBlock entryPoint: aMethodReference
	| entryPointReached |
	entryPointReached := false.
	"Iterate over all covered methods and mutate the current one 
	before executing aBlock"
	self coveredMethods do: [:reference | | wrapper |
		entryPointReached := entryPointReached or: [reference = aMethodReference].
		(entryPointReached and: [self shouldMutate: reference]) ifTrue: 
			[wrapper := SContractMutationWrapper 
				on: reference methodSymbol 
				inClass: reference actualClass.
			wrapper installClosestToMethod.
			[aBlock value: reference] ensure: [wrapper uninstall]]]! !

!SContractMutationEngine methodsFor: 'checking' stamp: 'or 1/31/2012 14:44:57.679'!
shouldMutate: aMethodReference

	^ (self isReturningSelf: aMethodReference) not and: 
	  [(self blacklistedMethods includes: aMethodReference) not]! !

Object subclass: #SExecutionPolicy
	instanceVariableNames: 'activeGroups preconditionsActive postconditionsActive invariantsActive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Core'!

SExecutionPolicy class
	instanceVariableNames: 'defaultPolicy'!

!SExecutionPolicy commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

SExecutionPolicy class
	instanceVariableNames: 'defaultPolicy'!

!SExecutionPolicy class methodsFor: 'instance creation' stamp: 'mp 5/14/2013 14:57:58.055'!
default
	^ defaultPolicy ifNil: [defaultPolicy := SExecutionPolicy executionDisabled]! !

!SExecutionPolicy class methodsFor: 'instance creation' stamp: 'ls 1/18/2012 11:49:26.422'!
executionDisabled
	| instance |
	instance := self new.
	instance
		activeGroups: {};
		preconditionsActive: false;
		postconditionsActive: false;
		invariantsActive: false.
	^ instance! !

!SExecutionPolicy class methodsFor: 'instance creation' stamp: 'ls 1/18/2012 11:49:44.272'!
executionEnabled
	| instance |
	instance := self new.
	instance 
		activeGroups: SProject allProjectNames;
		preconditionsActive: true;
		postconditionsActive: true;
		invariantsActive: true.
	^ instance! !

!SExecutionPolicy class methodsFor: 'public api' stamp: 'or 1/31/2012 14:54:49.04'!
nuke
	"remove policies of all processes so that they will get the new default policy"
	| processList |
	Smalltalk garbageCollectMost.
	processList := Process allSubInstances reject: [:each | each isTerminated].
	processList := WeakArray withAll: processList.
	processList do: [:each | each contractExecutionPolicy: nil].! !

!SExecutionPolicy methodsFor: 'accessing' stamp: 'ls 12/28/2011 17:07:49.928'!
activeGroups

	^ activeGroups! !

!SExecutionPolicy methodsFor: 'accessing' stamp: 'or 1/13/2012 14:33:59.346'!
activeGroups: aGroupOrCollection

	aGroupOrCollection isSymbol
		ifTrue: [activeGroups := (Set with: aGroupOrCollection)]
		ifFalse: [activeGroups := aGroupOrCollection]
! !

!SExecutionPolicy methodsFor: 'public api' stamp: 'ls 12/25/2011 18:49:07.367'!
ifNotActive: aBlock
	
	self isNotActive ifTrue: [^ aBlock value]! !

!SExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:35.084'!
invariantsActive
	"Answer the value of invariantsActive"

	^ invariantsActive! !

!SExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:35.133'!
invariantsActive: anObject
	"Set the value of invariantsActive"

	invariantsActive := anObject! !

!SExecutionPolicy methodsFor: 'public api' stamp: 'ls 12/25/2011 18:29:59.661'!
isActive
	
	^ self invariantsActive or: [self preconditionsActive] or: [self postconditionsActive]! !

!SExecutionPolicy methodsFor: 'public api' stamp: 'ls 12/25/2011 18:46:56.668'!
isNotActive
	
	^ self isActive not! !

!SExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:34.986'!
postconditionsActive
	"Answer the value of postconditionsActive"

	^ postconditionsActive! !

!SExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:35.034'!
postconditionsActive: anObject
	"Set the value of postconditionsActive"

	postconditionsActive := anObject! !

!SExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:34.878'!
preconditionsActive
	"Answer the value of preconditionsActive"

	^ preconditionsActive! !

!SExecutionPolicy methodsFor: 'accessing' stamp: 'ls 1/18/2012 11:48:34.934'!
preconditionsActive: anObject
	"Set the value of preconditionsActive"

	preconditionsActive := anObject! !

Object subclass: #SStateAnomaliesEvaluator
	instanceVariableNames: 'numberOfMutations mutationFactory pathMap results current'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Evaluation'!

!SStateAnomaliesEvaluator commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SStateAnomaliesEvaluator class methodsFor: 'instance creation' stamp: 'mp 6/10/2013 16:44:19.686'!
foriCal
	^ self foriCalWith: 100.! !

!SStateAnomaliesEvaluator class methodsFor: 'instance creation' stamp: 'mp 6/10/2013 16:44:07.57'!
foriCalWith: aNumber
	^ self new 
		numberOfMutations: aNumber;
		mutationFactory: (MEMutationFactory onMethodWrappers
							scope: (MEMutationScope new
								addPackage: 'ICal-Core-Abstract'; 
								addPackage: 'ICal-Core-IO'; 
								addPackage: 'ICal-Core-Model'; 
								addPackage: 'ICal-Core-Parser'; 
								addPackage: 'ICal-Core-Strategy';
								addPackage: 'ICal-Core-VCard-IO'; 
								addPackage: 'ICal-Core-VCard-Model'; 
								addPackage: 'ICal-Core-VCard-Parser'; 
								yourself))! !

!SStateAnomaliesEvaluator methodsFor: 'evaluation' stamp: 'mp 6/11/2013 09:30:25.086'!
checkForFailuresWithPathMap: aMutation
	self current: nil.
	self runAllTests.
	(self pathMap model hasFailures or: [self pathMap model hasErrors])
		ifTrue: [self current add: {aMutation method. aMutation class}.
				self checkForViolationWithPathMap: aMutation].
"	self results add: self current."! !

!SStateAnomaliesEvaluator methodsFor: 'evaluation' stamp: 'mp 6/11/2013 09:30:07.415'!
checkForViolationWithPathMap: aMutation
	| violations |
	[self faultLocalizationFlap switchExpandState.
	self runAllTestsWithContracts.
	violations := self pathMap model pathMap violations.
	self current add: violations.
	violations ifNotEmpty: [
		1 halt.
		Transcript 
			show: 'Mutation leads to ', violations keys size, ' violations!!.'; 
			cr]] ensure: [self faultLocalizationFlap switchExpandState].! !

!SStateAnomaliesEvaluator methodsFor: 'accessing' stamp: 'mp 6/10/2013 17:32:48.978'!
current

	^ current ifNil: [current := OrderedCollection new]! !

!SStateAnomaliesEvaluator methodsFor: 'accessing' stamp: 'mp 6/10/2013 17:32:37.404'!
current: anObject

	current := anObject! !

!SStateAnomaliesEvaluator methodsFor: 'public API' stamp: 'mp 6/10/2013 17:38:12.091'!
evaluate
	| mutation |
	Transcript clear.
	self numberOfMutations timesRepeat: [
		mutation := self mutationFactory randomMutation.
		[mutation doMutated: [self checkForFailuresWithPathMap: mutation]] 
			on: Error do: [:err | Transcript show: err asString; cr.]].
	self pathMap abandon.
	^ self results explore! !

!SStateAnomaliesEvaluator methodsFor: 'pathMap-commands' stamp: 'mp 6/10/2013 17:20:14.579'!
faultLocalizationFlap
	^ self pathMap model getFlap: #faultLocalization! !

!SStateAnomaliesEvaluator methodsFor: 'pathMap-commands' stamp: 'mp 6/10/2013 17:07:47.775'!
findButtonLabeled: aString
	^ self pathMap 
		findDeepSubmorphThat: [:morph | 
			morph class = PluggableButtonMorphPlus and: [morph label = aString]] 
		ifAbsent: nil ! !

!SStateAnomaliesEvaluator methodsFor: 'initialize-release' stamp: 'mp 6/10/2013 17:20:24.202'!
initialize
	super initialize.
	self pathMap: SPathMapTestRunner open.
	self pathMap model pathMap globalOptions switchVisualizeIntermediateSteps.! !

!SStateAnomaliesEvaluator methodsFor: 'accessing' stamp: 'mp 6/10/2013 16:41:24.351'!
mutationFactory

	^ mutationFactory! !

!SStateAnomaliesEvaluator methodsFor: 'accessing' stamp: 'mp 6/10/2013 16:41:24.357'!
mutationFactory: anObject

	mutationFactory := anObject! !

!SStateAnomaliesEvaluator methodsFor: 'accessing' stamp: 'mp 6/10/2013 16:41:24.336'!
numberOfMutations

	^ numberOfMutations! !

!SStateAnomaliesEvaluator methodsFor: 'accessing' stamp: 'mp 6/10/2013 16:41:24.344'!
numberOfMutations: anObject

	numberOfMutations := anObject! !

!SStateAnomaliesEvaluator methodsFor: 'accessing' stamp: 'mp 6/10/2013 16:54:08.211'!
pathMap

	^ pathMap! !

!SStateAnomaliesEvaluator methodsFor: 'accessing' stamp: 'mp 6/10/2013 16:54:08.218'!
pathMap: anObject

	pathMap := anObject! !

!SStateAnomaliesEvaluator methodsFor: 'accessing' stamp: 'mp 6/10/2013 17:28:27.141'!
results

	^ results ifNil: [results := OrderedCollection new]! !

!SStateAnomaliesEvaluator methodsFor: 'accessing' stamp: 'mp 6/10/2013 17:27:48.597'!
results: anObject

	results := anObject! !

!SStateAnomaliesEvaluator methodsFor: 'pathMap-commands' stamp: 'mp 6/10/2013 17:02:11.662'!
runAllTests
	(self findButtonLabeled: 'Run Selected') performAction.! !

!SStateAnomaliesEvaluator methodsFor: 'pathMap-commands' stamp: 'mp 6/10/2013 17:02:18.737'!
runAllTestsWithContracts
	(self findButtonLabeled: 'Run with Contracts') performAction.! !

Halt subclass: #SContractViolation
	instanceVariableNames: 'methodReference kindOf variableName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Exceptions'!

!SContractViolation commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SContractViolation class methodsFor: 'instance creation' stamp: 'mp 1/18/2013 13:32:08.644'!
on: aMethodReference kindOf: aSymbol
	^ self new
		methodReference: aMethodReference;
		kindOf: aSymbol! !

!SContractViolation class methodsFor: 'instance creation' stamp: 'mp 3/15/2012 12:21:16.646'!
signalOn: aMethodReference
	^ self signalOn: aMethodReference kindOf: #unknown.! !

!SContractViolation class methodsFor: 'instance creation' stamp: 'mp 3/30/2012 11:53:23.97'!
signalOn: aMethodReference kindOf: aSymbol
	^ (self on: aMethodReference kindOf: aSymbol)
		signal! !

!SContractViolation methodsFor: 'comparing' stamp: 'mp 3/21/2012 15:57:14.037'!
< other
	^ false! !

!SContractViolation methodsFor: 'comparing' stamp: 'mp 3/15/2012 14:19:53.704'!
= other
	self class = other class
		ifFalse: [^ false].
	^ (self methodReference = other methodReference and: [self kindOf = other kindOf])! !

!SContractViolation methodsFor: 'printing' stamp: 'mp 1/18/2013 15:54:37.36'!
description
	^ self messageText, 
	' (', self kindOf asString, 
	(self variableName ifNil: [''] ifNotNil: [' of ', self variableName]), 
	') in ', self methodReference asString! !

!SContractViolation methodsFor: 'comparing' stamp: 'mp 3/15/2012 14:22:57.59'!
hash
	^ self methodReference hash bitXor: self kindOf hash! !

!SContractViolation methodsFor: 'initialize-release' stamp: 'mp 3/30/2012 11:48:50.127'!
initialize
	super initialize.
	self messageText: 'Contract violation'! !

!SContractViolation methodsFor: 'testing' stamp: 'mp 3/15/2012 09:49:51.836'!
isContractViolation
	^ true! !

!SContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 15:02:25.908'!
isIncludesContractViolation
	^ false! !

!SContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 11:58:01.477'!
isNilContractViolation
	^ false! !

!SContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 13:12:47.231'!
isPropertyContractViolation
	^ false! !

!SContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 14:25:53.979'!
isRangeContractViolation
	^ false! !

!SContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 12:12:52.716'!
isSizeContractViolation
	^ false! !

!SContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 12:54:03.302'!
isSpellingContractViolation
	^ false! !

!SContractViolation methodsFor: 'testing' stamp: 'mp 3/15/2012 12:31:48.772'!
isTypeContractViolation
	^ false! !

!SContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:20:49.46'!
kindOf
	"Answer the value of kindOf"

	^ kindOf! !

!SContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:20:49.473'!
kindOf: anObject
	"Set the value of kindOf"

	kindOf := anObject! !

!SContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 09:51:58.039'!
methodReference
	"Answer the value of methodReference"

	^ methodReference! !

!SContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 09:51:58.051'!
methodReference: anObject
	"Set the value of methodReference"

	methodReference := anObject! !

!SContractViolation methodsFor: 'printing' stamp: 'mp 1/18/2013 14:39:39.968'!
shortDescription
	"required for labels in PathMap and PathFinder - see also SPMViolationMorph"
	self variableName 
		ifNotNil: [^ self shortKindOf, ' in ', self variableName,': ', self messageText].
	^ self shortKindOf, ': ', self messageText! !

!SContractViolation methodsFor: 'printing' stamp: 'mp 3/30/2012 12:09:55.089'!
shortKindOf
	^ self kindOf caseOf: {
		[#precondition] -> ['Pre'].
		[#invariant] -> ['Inv'].
		[#postcondition] -> ['Post'].
	}.! !

!SContractViolation methodsFor: 'accessing' stamp: 'mp 1/18/2013 13:31:34.289'!
variableName
	"Answer the value of variableName"

	^ variableName! !

!SContractViolation methodsFor: 'accessing' stamp: 'mp 1/18/2013 13:31:34.298'!
variableName: anObject
	"Set the value of variableName"

	variableName := anObject! !

SContractViolation subclass: #SIncludesContractViolation
	instanceVariableNames: 'expectedCollection givenObject property'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Exceptions'!

!SIncludesContractViolation commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SIncludesContractViolation methodsFor: 'comparing' stamp: 'mp 3/30/2012 15:01:45.519'!
= other
	self class = other class
		ifFalse: [^ false].
	^ self methodReference = other methodReference 
		and: [self kindOf = other kindOf]
		and: [self givenObject = other givenObject]
		and: [self expectedCollection = other expectedCollection]
		and: [self property = other property]! !

!SIncludesContractViolation methodsFor: 'printing' stamp: 'mp 5/14/2013 14:57:20.77'!
description
	| stream |
	stream := WriteStream on: ''.
	stream 
		nextPutAll: self messageText;
		nextPutAll: ' ('.
	self property 
		ifNil: [stream nextPutAll: self givenObject asString]
		ifNotNil: [stream 
					nextPutAll: self property asString;
					nextPutAll: ': ';
					nextPutAll: (SContract 
								getPropertyFrom: self givenObject 
								with: self property) asString].
	stream
		nextPutAll: ' is not included in ';
		nextPutAll: self expectedCollection asString;
		nextPutAll: ') ';
		nextPutAll: self kindOf asString;
		nextPutAll: (self variableName ifNil: [''] ifNotNil: [' of ', self variableName]);
		nextPutAll: ' fails in ';
		nextPutAll: self methodReference asString.
	^ stream contents
	! !

!SIncludesContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:54:28.993'!
expectedCollection
	"Answer the value of expectedCollection"

	^ expectedCollection! !

!SIncludesContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:54:29.005'!
expectedCollection: anObject
	"Set the value of expectedCollection"

	expectedCollection := anObject! !

!SIncludesContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:54:29.016'!
givenObject
	"Answer the value of givenObject"

	^ givenObject! !

!SIncludesContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:54:29.028'!
givenObject: anObject
	"Set the value of givenObject"

	givenObject := anObject! !

!SIncludesContractViolation methodsFor: 'comparing' stamp: 'mp 3/30/2012 15:01:05.453'!
hash
	^ ((super hash bitXor: self givenObject hash) bitXor: self expectedCollection hash) 		bitXor: self property hash! !

!SIncludesContractViolation methodsFor: 'initialize-release' stamp: 'mp 3/30/2012 14:55:37.922'!
initialize
	super initialize.
	self messageText: 'Includes violation'! !

!SIncludesContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 15:02:39.115'!
isIncludesContractViolation
	^ true! !

!SIncludesContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:54:29.039'!
property
	"Answer the value of property"

	^ property! !

!SIncludesContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:54:29.05'!
property: anObject
	"Set the value of property"

	property := anObject! !

!SIncludesContractViolation methodsFor: 'printing' stamp: 'mp 5/14/2013 14:57:24.007'!
shortDescription
	| stream |
	stream := WriteStream on: ''.
	stream 
		nextPutAll: super shortDescription;
		nextPutAll: ' >> '.
	self property 
		ifNil: [stream nextPutAll: self givenObject asString]
		ifNotNil: [stream nextPutAll: (SContract 
								getPropertyFrom: self givenObject 
								with: self property) asString].
	stream
			nextPutAll: ' not included in ';
			nextPutAll: self expectedCollection asString.
	^ stream contents
	! !

SContractViolation subclass: #SNilContractViolation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Exceptions'!

!SNilContractViolation commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SNilContractViolation methodsFor: 'initialize-release' stamp: 'mp 3/30/2012 11:49:52.148'!
initialize
	super initialize.
	self messageText: 'Nil violation'! !

!SNilContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 11:57:47.715'!
isNilContractViolation
	^ true! !

SContractViolation subclass: #SPropertyContractViolation
	instanceVariableNames: 'expected givenProperty expectedProperty'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Exceptions'!

!SPropertyContractViolation commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SPropertyContractViolation methodsFor: 'comparing' stamp: 'mp 3/30/2012 13:13:36.266'!
= other
	self class = other class
		ifFalse: [^ false].
	^ self methodReference = other methodReference 
		and: [self kindOf = other kindOf]
		and: [self givenProperty = other givenProperty]
		and: [self expectedProperty = other expectedProperty]
		and: [self expected = self expected]! !

!SPropertyContractViolation methodsFor: 'printing' stamp: 'mp 1/18/2013 15:58:33.903'!
description
	| stream |
	stream := WriteStream on: ''.
	stream 
		nextPutAll: self messageText;
		nextPutAll: ' (Expected to be '.
	self expected
		ifFalse: [stream nextPutAll: 'not '].
	stream
		nextPutAll: self expectedProperty asString;
		nextPutAll: ') ';
		nextPutAll: self kindOf asString;
		nextPutAll: (self variableName ifNil: [''] ifNotNil: [' of ', self variableName]);
		nextPutAll: ' fails in ';
		nextPutAll: self methodReference asString.
	^ stream contents! !

!SPropertyContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 13:03:50.13'!
expected
	"Answer the value of expected"

	^ expected! !

!SPropertyContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 13:03:50.142'!
expected: anObject
	"Set the value of expected"

	expected := anObject! !

!SPropertyContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 13:09:23.656'!
expectedProperty
	"Answer the value of expectedProperty"

	^ expectedProperty! !

!SPropertyContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 13:09:23.669'!
expectedProperty: anObject
	"Set the value of expectedProperty"

	expectedProperty := anObject! !

!SPropertyContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 13:10:06.961'!
givenProperty
	"Answer the value of givenProperty"

	^ givenProperty! !

!SPropertyContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 13:10:06.973'!
givenProperty: anObject
	"Set the value of givenProperty"

	givenProperty := anObject! !

!SPropertyContractViolation methodsFor: 'comparing' stamp: 'mp 3/30/2012 13:14:17.108'!
hash
	^ ((super hash bitXor: self givenProperty hash) bitXor: self expectedProperty hash) 		bitXor: self expected hash! !

!SPropertyContractViolation methodsFor: 'initialize-release' stamp: 'mp 3/30/2012 13:10:41.748'!
initialize
	super initialize.
	self messageText: 'Property violation'! !

!SPropertyContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 13:12:54.83'!
isPropertyContractViolation
	^ true! !

!SPropertyContractViolation methodsFor: 'printing' stamp: 'mp 3/30/2012 13:28:23.432'!
shortDescription
	| stream | 
	stream := WriteStream on: ''.
	stream
		nextPutAll: super shortDescription;
		nextPutAll: ' >> Expected '.
	self expected
		ifFalse: [stream nextPutAll: 'not '].		
	stream
		nextPutAll: self expectedProperty asString.
	^ stream contents		! !

SContractViolation subclass: #SRangeContractViolation
	instanceVariableNames: 'expectedMin expectedMax givenObject property'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Exceptions'!

!SRangeContractViolation commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SRangeContractViolation methodsFor: 'comparing' stamp: 'mp 3/30/2012 14:29:28.518'!
= other
	self class = other class
		ifFalse: [^ false].
	^ (self methodReference = other methodReference 
		and: [self kindOf = other kindOf]
		and: [self givenObject = other givenObject]
		and: [self expectedMin = other expectedMin]
		and: [self expectedMax = other expectedMax])	
		and: [self property = other property]! !

!SRangeContractViolation methodsFor: 'printing' stamp: 'mp 1/18/2013 15:58:17.505'!
description
	| stream |
	stream := WriteStream on: ''.
	stream 
		nextPutAll: self messageText;
		nextPutAll: ' ('.
	self property 
		ifNil: [stream nextPutAll: self givenObject asString]
		ifNotNil: [stream 
					nextPutAll: self property asString;
					nextPutAll: ': ';
					nextPutAll: (self givenObject perform: self property) asString].
	stream
		nextPutAll: ' not between ';
		nextPutAll: self expectedMin asString;
		nextPutAll: ' and ';
		nextPutAll: self expectedMax asString;
		nextPutAll: ') ';
		nextPutAll: self kindOf asString;
		nextPutAll: (self variableName ifNil: [''] ifNotNil: [' of ', self variableName]);
		nextPutAll: ' fails in ';
		nextPutAll: self methodReference asString.
	^ stream contents
	! !

!SRangeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:28:24.053'!
expectedMax
	"Answer the value of expectedMax"

	^ expectedMax! !

!SRangeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:28:24.064'!
expectedMax: anObject
	"Set the value of expectedMax"

	expectedMax := anObject! !

!SRangeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:28:24.029'!
expectedMin
	"Answer the value of expectedMin"

	^ expectedMin! !

!SRangeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:28:24.041'!
expectedMin: anObject
	"Set the value of expectedMin"

	expectedMin := anObject! !

!SRangeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:24:11.481'!
givenObject
	"Answer the value of givenObject"

	^ givenObject! !

!SRangeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:24:11.492'!
givenObject: anObject
	"Set the value of givenObject"

	givenObject := anObject! !

!SRangeContractViolation methodsFor: 'comparing' stamp: 'mp 3/30/2012 14:29:57.412'!
hash
	^ (((super hash bitXor: self givenObject hash) bitXor: self expectedMin hash) bitXor: 		self expectedMax hash) bitXor: self property hash! !

!SRangeContractViolation methodsFor: 'initialize-release' stamp: 'mp 3/30/2012 14:24:25.559'!
initialize
	super initialize.
	self messageText: 'Range violation'! !

!SRangeContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 14:26:01.53'!
isRangeContractViolation
	^ true! !

!SRangeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:24:11.504'!
property
	"Answer the value of property"

	^ property! !

!SRangeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 14:24:11.515'!
property: anObject
	"Set the value of property"

	property := anObject! !

!SRangeContractViolation methodsFor: 'printing' stamp: 'mp 3/30/2012 14:51:18.262'!
shortDescription
	| stream |
	stream := WriteStream on: ''.
	stream 
		nextPutAll: super shortDescription;
		nextPutAll: ' >> '.
	self property 
		ifNil: [stream nextPutAll: self givenObject asString]
		ifNotNil: [stream nextPutAll: (self givenObject perform: self property) asString].
	stream
			nextPutAll: ' not between ';
			nextPutAll: self expectedMin asString;
			nextPutAll: ' and ';
			nextPutAll: self expectedMax asString.
	^ stream contents
		! !

SContractViolation subclass: #SSizeContractViolation
	instanceVariableNames: 'givenSize expectedSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Exceptions'!

!SSizeContractViolation commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SSizeContractViolation methodsFor: 'comparing' stamp: 'mp 3/30/2012 12:04:50.213'!
= other
	self class = other class
		ifFalse: [^ false].
	^ self methodReference = other methodReference 
		and: [self kindOf = other kindOf]
		and: [self givenSize = other givenSize]
		and: [self expectedSize = other expectedSize]! !

!SSizeContractViolation methodsFor: 'printing' stamp: 'mp 1/18/2013 15:56:48.942'!
description
	^ self messageText, 
	' (Size: ', self givenSize asString,
	' > ', self expectedSize asString, ' elements) ',
	self kindOf asString, 
	(self variableName ifNil: [''] ifNotNil: [' of ', self variableName]), 
	' fails in ', self methodReference asString! !

!SSizeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 11:59:44.251'!
expectedSize
	"Answer the value of expectedSize"

	^ expectedSize! !

!SSizeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 11:59:44.262'!
expectedSize: anObject
	"Set the value of expectedSize"

	expectedSize := anObject! !

!SSizeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 11:59:44.228'!
givenSize
	"Answer the value of givenSize"

	^ givenSize! !

!SSizeContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 11:59:44.24'!
givenSize: anObject
	"Set the value of givenSize"

	givenSize := anObject! !

!SSizeContractViolation methodsFor: 'comparing' stamp: 'mp 3/30/2012 12:05:07.849'!
hash
	^ (super hash bitXor: self givenSize hash) bitXor: self expectedSize hash! !

!SSizeContractViolation methodsFor: 'initialize-release' stamp: 'mp 3/30/2012 12:01:10.648'!
initialize
	super initialize.
	self messageText: 'Size violation'! !

!SSizeContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 12:13:00.926'!
isSizeContractViolation
	^ true! !

!SSizeContractViolation methodsFor: 'printing' stamp: 'mp 3/30/2012 12:11:27.659'!
shortDescription
	^ super shortDescription,
		' >> ', self givenSize asString,
		' > ', self expectedSize asString! !

SContractViolation subclass: #SSpellingContractViolation
	instanceVariableNames: 'givenString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Exceptions'!

!SSpellingContractViolation commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!SSpellingContractViolation methodsFor: 'comparing' stamp: 'mp 3/30/2012 12:52:45.015'!
= other
	self class = other class
		ifFalse: [^ false].
	^ self methodReference = other methodReference 
		and: [self kindOf = other kindOf]
		and: [self givenString = other givenString]! !

!SSpellingContractViolation methodsFor: 'printing' stamp: 'mp 1/18/2013 15:56:28.295'!
description
	^ self messageText, 
	' (Spelling incorrect: ', self givenString, ') ',
	self kindOf asString, 
	(self variableName ifNil: [''] ifNotNil: [' of ', self variableName]), 
	' fails in ', self methodReference asString! !

!SSpellingContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 12:51:55.878'!
givenString
	"Answer the value of givenString"

	^ givenString! !

!SSpellingContractViolation methodsFor: 'accessing' stamp: 'mp 3/30/2012 12:51:55.89'!
givenString: anObject
	"Set the value of givenString"

	givenString := anObject! !

!SSpellingContractViolation methodsFor: 'comparing' stamp: 'mp 3/30/2012 12:53:18.358'!
hash
	^ super hash bitXor: self givenString hash! !

!SSpellingContractViolation methodsFor: 'initialize-release' stamp: 'mp 3/30/2012 12:53:36.46'!
initialize
	super initialize.
	self messageText: 'Spelling violation'! !

!SSpellingContractViolation methodsFor: 'testing' stamp: 'mp 3/30/2012 12:53:53.977'!
isSpellingContractViolation
	^ true! !

!SSpellingContractViolation methodsFor: 'printing' stamp: 'mp 3/30/2012 12:55:47.016'!
shortDescription
	^ super shortDescription,
		' >> ', self givenString asString! !

SContractViolation subclass: #STypeContractViolation
	instanceVariableNames: 'givenType expectedType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Path-Contracts-Exceptions'!

!STypeContractViolation commentStamp: '<historical>' prior: 0!
All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de

All rights reserved. 

Copyright (c) 2009-to present. 
Michael Perscheid, Software Architecture Group, 
Hasso-Plattner-Institute, University of Potsdam.

Contact: michael.perscheid@gmail.com
Website: www.michaelperscheid.de!

!STypeContractViolation methodsFor: 'comparing' stamp: 'mp 3/15/2012 14:25:29.677'!
= other
	self class = other class
		ifFalse: [^ false].
	^ self methodReference = other methodReference 
		and: [self kindOf = other kindOf]
		and: [self givenType = other givenType]
		and: [self expectedType = other expectedType]! !

!STypeContractViolation methodsFor: 'printing' stamp: 'mp 1/18/2013 15:55:46.948'!
description
	^ self messageText, 
	' (Given: ', self givenType asString,
	' Expected: ', self expectedType asString, ') ',
	self kindOf asString, 
	(self variableName ifNil: [''] ifNotNil: [' of ', self variableName]), 
	' fails in ', self methodReference asString! !

!STypeContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:28:04.62'!
expectedType
	"Answer the value of expectedType"

	^ expectedType! !

!STypeContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:28:04.631'!
expectedType: anObject
	"Set the value of expectedType"

	expectedType := anObject! !

!STypeContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:28:04.596'!
givenType
	"Answer the value of givenType"

	^ givenType! !

!STypeContractViolation methodsFor: 'accessing' stamp: 'mp 3/15/2012 12:28:04.608'!
givenType: anObject
	"Set the value of givenType"

	givenType := anObject! !

!STypeContractViolation methodsFor: 'comparing' stamp: 'mp 3/15/2012 14:26:39.993'!
hash
	^ (super hash bitXor: self givenType hash) bitXor: self expectedType hash! !

!STypeContractViolation methodsFor: 'testing' stamp: 'mp 3/15/2012 12:31:54.322'!
isTypeContractViolation
	^ true! !

!STypeContractViolation methodsFor: 'printing' stamp: 'mp 3/30/2012 12:11:47.893'!
shortDescription
	^ super shortDescription,
		' >> ', self givenType asString,
		' !!= ', self expectedType asString! !
SContractExecutionPolicyRegistry initialize!
SContractInvariantRegistry initialize!
